package net.anotheria.anouitest.core;

import com.google.common.util.concurrent.MoreExecutors;
import net.anotheria.anouitest.page.PageFactory;
import net.anotheria.anouitest.settings.GeneralConfig;
import net.anotheria.anouitest.step.BeforeAfterStep;
import net.anotheria.anouitest.step.Step;
import org.jbehave.core.configuration.Configuration;
import org.jbehave.core.embedder.Embedder;
import org.jbehave.core.failures.FailingUponPendingStep;
import org.jbehave.core.io.StoryFinder;
import org.jbehave.core.reporters.Format;
import org.jbehave.core.reporters.StoryReporterBuilder;
import org.jbehave.core.steps.InjectableStepsFactory;
import org.jbehave.core.steps.InstanceStepsFactory;
import org.jbehave.core.steps.ParameterConverters;
import org.jbehave.core.steps.SilentStepMonitor;
import org.jbehave.web.selenium.ContextView;
import org.jbehave.web.selenium.SeleniumConfiguration;
import org.jbehave.web.selenium.SeleniumContext;
import org.jbehave.web.selenium.SeleniumStepMonitor;
import org.jbehave.web.selenium.WebDriverProvider;
import org.jbehave.web.selenium.WebDriverScreenshotOnFailure;
import org.jbehave.web.selenium.WebDriverSteps;
import org.reflections.Reflections;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Acceptance runner. Run all found tests.
 *
 * @author ivanbatura
 */
public class AcceptanceRunner extends Embedder {
    /**
     * Driver Provider, stores driver.
     */
    private final WebDriverProvider driverProvider;
    /**
     * Setups and tier down steps.
     */
    private final WebDriverSteps lifecycleSteps;
    /**
     * Page Factory.
     */
    private final PageFactory pages;
    /**
     * Selenium Content.
     */
    private final SeleniumContext context;

    /**
     * View content in order to no show any dialog frames.
     */
    private final ContextView contextView;

    /**
     * Converters.
     */
    private ParameterConverters converters;

    /**
     * Symbol for excluding int testing scenarios by Meta.
     */
    private static final String EXCLUDE = "-";

    /**
     * Symbol for including in testing scenarios by Meta.
     */
    private static final String INCLUDE = "+";

    /**
     * General config.
     */
    private static final GeneralConfig config = GeneralConfig.getInstance();

    public AcceptanceRunner() throws UITestException {
        //init
        driverProvider = new MultiWebDriverProvider(Arrays.asList(config.getRunnableDrivers()), MultiWebDriverProvider.WebDriverChangingMode.MANUAL);
        pages = new PageFactory(driverProvider);
        lifecycleSteps = new BeforeAfterStep(driverProvider, pages);
        context = new SeleniumContext();
        contextView = new ContextView.NULL();
        converters = initConverters();

        //filter
        List<String> metaFilter = new ArrayList<String>();
        for (String s : config.getExcludedScenariosByMeta())
            metaFilter.add(EXCLUDE + s);
        for (String s : config.getIncludedScenariosByMeta())
            metaFilter.add(INCLUDE + s);
        if (!metaFilter.isEmpty())
            useMetaFilters(metaFilter);

        useExecutorService(MoreExecutors.sameThreadExecutor());
        useStoryRunner(new MultiStoryRunner());
//		configuredEmbedder().useEmbedderFailureStrategy(new Embedder.ThrowingRunningStoriesFailed());
    }

    @Override
    public Configuration configuration() {
        return new SeleniumConfiguration()
                .useSeleniumContext(context)
                .useWebDriverProvider(driverProvider)
                .useStepMonitor(new SeleniumStepMonitor(contextView, context, new SilentStepMonitor()))
                .useParameterConverters(converters)
                .useFailureStrategy(new FailingUponPendingStep())
                .useStoryReporterBuilder(new StoryReporterBuilder()
                        .withDefaultFormats()
                        .withFormats(Format.CONSOLE, Format.HTML, Format.XML, Format.STATS));
    }

    @Override
    public InjectableStepsFactory stepsFactory() {
        Configuration configuration = configuration();
        List<Object> steps = new ArrayList<Object>();
        steps.add(lifecycleSteps);
        steps.add(new WebDriverScreenshotOnFailure(driverProvider, configuration.storyReporterBuilder()));
        for (Class<?> type : new Reflections("").getSubTypesOf(Step.class)) {
            if (!type.isInterface()) {
                try {
                    Constructor ctor = type.getDeclaredConstructor(PageFactory.class);
                    steps.add(ctor.newInstance(pages));
                } catch (NoSuchMethodException e) {
                    throw new RuntimeException("Can not added step to the runner", e);
                } catch (InvocationTargetException e) {
                    throw new RuntimeException("Can not added step to the runner", e);
                } catch (InstantiationException e) {
                    throw new RuntimeException("Can not added step to the runner", e);
                } catch (IllegalAccessException e) {
                    throw new RuntimeException("Can not added step to the runner", e);
                }
            }
        }
        return new InstanceStepsFactory(configuration, steps);
    }

    /**
     * Run stories.
     *
     * @param absolutePathForStories path for the story files
     */
    public void run(String absolutePathForStories) {
        runStoriesAsPaths(storyPaths(absolutePathForStories));
    }

    /**
     * Get all stories.
     *
     * @param absolutePathForStories path for the story files
     * @return List of the stories.
     */
    protected List<String> storyPaths(String absolutePathForStories) {
        String includePattern = "**/*.story";
        String excludePattern = "**/excluded*.story";
        return new StoryFinder().findPaths(absolutePathForStories, Arrays.asList(includePattern), Arrays.asList(excludePattern));
    }

    /**
     * Init all converters.
     *
     * @return all converters
     * @throws UITestException if error
     */
    private ParameterConverters initConverters() throws UITestException {
        ParameterConverters result = new ParameterConverters(true);
        result.addConverters(new ParameterConverters.DateConverter(new SimpleDateFormat(config.getDefaultDateFormat())));
        for (Class<?> type : new Reflections("").getSubTypesOf(ParameterConverters.ParameterConverter.class)) {
            if (!type.isInterface()) {
                try {
                    Constructor ctor = type.getDeclaredConstructor();
                    result.addConverters((ParameterConverters.ParameterConverter) ctor.newInstance());
                } catch (NoSuchMethodException e) {
                    throw new UITestException("Can not added converter to converters", e);
                } catch (InvocationTargetException e) {
                    throw new UITestException("Can not added converter to converters", e);
                } catch (InstantiationException e) {
                    throw new UITestException("Can not added converter to converters", e);
                } catch (IllegalAccessException e) {
                    throw new UITestException("Can not added converter to converters", e);
                }
            }
        }
        return result;
    }

}