package net.anotheria.anouitest.core;

import com.gargoylesoftware.htmlunit.BrowserVersion;
import net.anotheria.util.StringUtils;
import org.jbehave.web.selenium.DelegatingWebDriverProvider;
import org.openqa.selenium.ScreenOrientation;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.android.AndroidDriver;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.firefox.FirefoxDriver;
import org.openqa.selenium.htmlunit.HtmlUnitDriver;
import org.openqa.selenium.ie.InternetExplorerDriver;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import static java.lang.Boolean.parseBoolean;

/**
 * WebDiver provider, that support changing web driver without restart.
 *
 * @author ivanbatura
 */
public class MultiWebDriverProvider extends DelegatingWebDriverProvider {

    /**
     * Web driver for running.
     */
    private final List<RunnableBrowser> runnableDrivers;

    /**
     * Web driver changing mode while running.
     */
    private final WebDriverChangingMode mode;

    /**
     * Web driver parameters map.
     */
    private final Map<String, String> webDriverParameters;

    /**
     * Number of the active web driver.
     */
    private int activeBrowser;

    public enum WebDriverChangingMode {
        /**
         * Manual changing mode.
         */
        MANUAL,
        /**
         * Auto changing mode, round robin mode.
         */
        AUTO_ROUND_ROBIN
    }

    /**
     * Constructor.
     *
     * @param runnableDrivers List of {@link RunnableBrowser}
     */
    public MultiWebDriverProvider(List<RunnableBrowser> runnableDrivers) {
        this(runnableDrivers, null, null);
    }

    /**
     * Constructor.
     *
     * @param runnableDrivers List of {@link RunnableBrowser}
     * @param mode            {@link WebDriverChangingMode}
     */
    public MultiWebDriverProvider(List<RunnableBrowser> runnableDrivers, WebDriverChangingMode mode) {
        this(runnableDrivers, mode, null);
    }

    /**
     * Constructor.
     *
     * @param runnableDrivers     List of {@link RunnableBrowser}
     * @param mode                {@link WebDriverChangingMode}
     * @param webDriverParameters web driver parameters
     */
    public MultiWebDriverProvider(List<RunnableBrowser> runnableDrivers, WebDriverChangingMode mode, String webDriverParameters) {
        this.runnableDrivers = runnableDrivers != null ? runnableDrivers : new ArrayList<RunnableBrowser>(Arrays.asList(RunnableBrowser.HTMLUNIT));
        this.mode = mode != null ? mode : WebDriverChangingMode.MANUAL;
        this.webDriverParameters = webDriverParameters != null ? StringUtils.tokenize2map(webDriverParameters, ';', '=') : new HashMap<String, String>();
        activeBrowser = 0;
    }

    /**
     * init web driver.
     */
    public void initialize() {
        delegate.set(createDriver(runnableDrivers.get(activeBrowser)));
        if (WebDriverChangingMode.AUTO_ROUND_ROBIN.equals(mode))
            nextDriver();
    }

    /**
     * use next wen driver.
     */
    public void nextDriver() {
        activeBrowser++;
        if (activeBrowser >= runnableDrivers.size())
            activeBrowser = 0;
    }

    /**
     * get current web driver browser.
     *
     * @return {@link RunnableBrowser}
     */
    public RunnableBrowser getCurrentBrowser() {
        return runnableDrivers.get(activeBrowser);
    }

    /**
     * Create web driver.
     *
     * @param browser {@link RunnableBrowser}
     * @return {@link WebDriver}
     */
    private WebDriver createDriver(RunnableBrowser browser) {
        switch (browser) {
            case ANDROID:
                return createAndroidDriver();
            case CHROME:
                return createChromeDriver();
            case FIREFOX:
                return createFirefoxDriver();
            case HTMLUNIT:
            default:
                return createHtmlUnitDriver();
            case IE:
                return createInternetExplorerDriver();
        }
    }

    /**
     * Create Android web driver.
     *
     * @return {@link WebDriver}
     */
    protected WebDriver createAndroidDriver() {
        try {
            String url = getParameter("webdriver.android.url", "http://localhost:8080/hub");
            ScreenOrientation screenOrientation = ScreenOrientation.valueOf(getParameter(
                    "webdriver.android.screenOrientation", "portrait").toUpperCase(usingLocale()));
            AndroidDriver driver = new AndroidDriver(url);
            driver.rotate(screenOrientation);
            return driver;
        } catch (MalformedURLException e) {
            throw new UnsupportedOperationException(e);
        }
    }

    /**
     * Create Chrome web driver.
     *
     * @return {@link ChromeDriver}
     */
    protected ChromeDriver createChromeDriver() {
        return new ChromeDriver();
    }

    /**
     * Create Firefox web drover.
     *
     * @return {@link FirefoxDriver}
     */
    protected FirefoxDriver createFirefoxDriver() {
        return new FirefoxDriver();
    }

    protected WebDriver createHtmlUnitDriver() {
        HtmlUnitDriver driver = new HtmlUnitDriver(BrowserVersion.FIREFOX_3_6);
        boolean javascriptEnabled = parseBoolean(getParameter("webdriver.htmlunit.javascriptEnabled", "true"));
        driver.setJavascriptEnabled(javascriptEnabled);
        return driver;
    }

    /**
     * Create Internet Explorer web driver .
     *
     * @return {@link InternetExplorerDriver}
     */
    protected InternetExplorerDriver createInternetExplorerDriver() {
        return new InternetExplorerDriver();
    }

    /**
     * Using default local.
     *
     * @return {@link Locale}
     */
    protected Locale usingLocale() {
        return Locale.getDefault();
    }

    /**
     * get relevant parameter from {@code webDriverParameters}
     *
     * @param key          Key
     * @param defaultValue default value if not found by {@code key}
     * @return value
     */
    private String getParameter(String key, String defaultValue) {
        String result = webDriverParameters.get(key);
        if (result == null)
            result = defaultValue;

        return result;
    }
}
