package net.anotheria.anouitest.page;

import net.anotheria.anouitest.core.MultiWebDriverProvider;
import net.anotheria.anouitest.core.RunnableBrowser;
import net.anotheria.anouitest.settings.GeneralConfig;
import org.jbehave.web.selenium.WebDriverPage;
import org.jbehave.web.selenium.WebDriverProvider;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedCondition;
import org.openqa.selenium.support.ui.Wait;
import org.openqa.selenium.support.ui.WebDriverWait;

import java.util.ArrayList;
import java.util.List;

/**
 * All classes that works with selenium should be extended from this class.
 *
 * @author ivanbatura
 */
public abstract class Page extends WebDriverPage {
    /**
     * General config.
     */
    private static final GeneralConfig config = GeneralConfig.getInstance();

    public void clickActiveDialogButton(String button) {
        WebElement dialog = getDriverProvider().get().findElement(By.cssSelector(".ui-dialog"));
        if (dialog.isDisplayed()) {
            List<WebElement> buttons = dialog.findElements(By.className("ui-button"));
            for (WebElement buttonElement : buttons) {
                if (button.equals(buttonElement.findElement(By.className("ui-button-text")).getText())) {
                    buttonElement.click();
                    return;
                }
            }
        }
    }

    /**
     * Constrictor.
     *
     * @param driverProvider {@link WebDriverPage}
     */
    public Page(WebDriverProvider driverProvider) {
        super(driverProvider);
    }

    /**
     * Init all page elements.
     */
    public void initElements() {
        org.openqa.selenium.support.PageFactory.initElements(getDriverProvider().get(), this);
    }

    /**
     * Open base url.
     */
    public void openBaseUrl() {
        get(getBaseUrl());
    }

    /**
     * Return runnable browser.
     *
     * @return {@link RunnableBrowser}
     */
    protected RunnableBrowser getRunningBrowser() {
        return ((MultiWebDriverProvider) getDriverProvider()).getCurrentBrowser();
    }

    /**
     * Get base url.
     *
     * @return base url
     */
    public String getBaseUrl() {
        return config.getBaseUrl();
    }

    /**
     * get page name.
     *
     * @return Page name
     */
    public String getPageName() {
        return this.getClass().getSimpleName();
    }

    /**
     * Find element that was created with javascript.
     *
     * @param by {@link By} selector
     * @return {@link WebElement} or null if no element was created and timeout has passed
     */
    public WebElement findAjaxElement(By by) {
        Wait<WebDriver> wait = new WebDriverWait(getDriverProvider().get(), config.getAjaxElementTimeout());
        return wait.until(visibilityOfElement(by));
    }

    /**
     * Find elements that was created with javascript.
     *
     * @param by {@link By} selector
     * @return List of {@link WebElement} or empty list if no element was created and timeout has passed
     */
    public List<WebElement> findAjaxElements(By by) {
        Wait<WebDriver> wait = new WebDriverWait(getDriverProvider().get(), config.getAjaxElementTimeout());
        List<WebElement> elements = wait.until(visibilityOfElements(by));
        return elements!=null ? elements : new ArrayList<WebElement>();
    }

    /**
     * Check if element has been crated ot not.
     *
     * @param by {@link By}
     * @return {@link ExpectedCondition}
     */
    private ExpectedCondition<WebElement> visibilityOfElement(final By by) {
        return new ExpectedCondition<WebElement>() {
            public WebElement apply(WebDriver driver) {
                WebElement element = driver.findElement(by);
                return element.isDisplayed() ? element : null;
            }
        };
    }

    /**
     * Check if all elements have been crated ot not.
     *
     * @param by {@link By}
     * @return {@link ExpectedCondition}
     */
    private ExpectedCondition<List<WebElement>> visibilityOfElements(final By by) {
        return new ExpectedCondition<List<WebElement>>() {
            public List<WebElement> apply(WebDriver driver) {
                List<WebElement> elements = driver.findElements(by);
                return elements.size() > 0 ? elements : null;
            }
        };
    }
}