package net.anotheria.anouitest.step;

import net.anotheria.anouitest.page.PageFactory;
import net.anotheria.anouitest.settings.EmbeddedServerConfig;
import net.anotheria.anouitest.webserver.EmbeddedTomcat;
import net.anotheria.anouitest.webserver.WebServerProvider;
import net.anotheria.anouitest.webserver.WebServerWorker;
import org.jbehave.core.annotations.AfterScenario;
import org.jbehave.core.annotations.AfterStories;
import org.jbehave.core.annotations.BeforeScenario;
import org.jbehave.core.annotations.BeforeStories;
import org.jbehave.web.selenium.WebDriverProvider;
import org.jbehave.web.selenium.WebDriverSteps;

import java.io.File;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Setup and tier down methods.
 *
 * @author ivanbatura
 */
public class BeforeAfterStep extends WebDriverSteps {
    /**
     * Page Factory.
     */
    private PageFactory pages;

    /**
     * Embedded config.
     */
    private final EmbeddedServerConfig config = EmbeddedServerConfig.getInstance();

    /**
     * Web server provider.
     */
    private WebServerProvider webServerProvider;

    /**
     * Constructor.
     *
     * @param driverProvider {@link WebDriverProvider}
     * @param pages          {@link PageFactory}
     */
    public BeforeAfterStep(WebDriverProvider driverProvider, PageFactory pages) {
        super(driverProvider);
        this.pages = pages;
    }

    /**
     * Constructor.
     *
     * @param driverProvider {@link WebDriverProvider}
     */
    public BeforeAfterStep(WebDriverProvider driverProvider) {
        super(driverProvider);
    }

    /**
     * Executed before each Scenario.
     */
    @BeforeScenario
    public void beforeScenario() {
        driverProvider.initialize();

    }

    /**
     * Executed after each Scenario.
     */
    @AfterScenario
    public void afterScenarios() {
        pages.reset();
        driverProvider.end();
    }

    /**
     * Executed before each Story.
     *
     * @throws Exception if error
     */
    @BeforeStories
    public void before() throws Exception {
        if (config.isUseEmbeddedServer())
            setupServer();

    }

    /**
     * Executed after each Story.
     *
     * @throws Exception if error
     */
    @AfterStories
    public void after() throws Exception {
        if (config.isUseEmbeddedServer())
            tierDownServer();
    }

    /**
     * Setup embedded server.
     */
    private void setupServer() {
        EmbeddedTomcat embeddedTomcat = new EmbeddedTomcat();
        embeddedTomcat.deployWar(config.getWebAppName(), new File(config.getPathToWar()));
        webServerProvider = new WebServerProvider(embeddedTomcat);
        if (config.isLunchServerInOtherThread()) {
            ExecutorService executorService = Executors.newSingleThreadExecutor();
            executorService.execute(new WebServerWorker(webServerProvider));
        } else {
            webServerProvider.startServer();
        }
    }

    /**
     * Stop embedded server.
     */
    private void tierDownServer() {
        webServerProvider.stopServer();
    }
}